# /**
#
# \name{qualityDiagnostics}
#
# \title{qualityDiagnostics}
#
# \alias{qualityDiagnostics}
#
# \keyword{}
#
# \description{}
#
# \value{}
#
#
# \arguments{
#  \item{exprSetArrayObject}{required}
#  \item{arrayDataObject}{required}
#  \item{normObject}{optional}
#  \item{qualityParametersList}{required}
#  \item{slideLabels}{optional}
#  \item{savegraphicspath}{optional}
#  \item{plotOutput}{character string specifying output plotOutput;
#        either "screen" or "pdf"; default: "screen"}
# }
# 
# \details{}
#
# \seealso{\code{\link{}}}
#
# \examples{
#	}
#
# \keyword{}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
# */

qualityDiagnostics <- function(exprSetArrayObject,arrayDataObject,normObject,qualityParametersList, slideLabels, savegraphicspath, plotOutput="screen"){

  
  if( ! plotOutput %in% c("screen","pdf") ){
    warning(" unknown plotOutput in qualityDiagnostics", call.=FALSE)
    return()
  }

  # so far even in case of plotOutput=="screen"
  # most of the graphics are copied to pdf file anyway
  device <- pdf
  suffix <- "pdf"
  
  if( missing(exprSetArrayObject) |
      missing(arrayDataObject) |
      missing(qualityParametersList) ){
    stop(" missing arguments in qualityDiagnostics ")
  }
  
  if( missing(slideLabels) ){
    slideLabels <- 1:dim(exprs(getExprSetDiff(exprSetArrayObject)))[2]
  }
  
  # slide distances 
  title <- "slideDistances"
  if( ! missing(savegraphicspath) ){
    fileName <- file.path(savegraphicspath,paste(title,".",suffix,sep=""))
  }else{
    fileName <- paste(title,".",suffix,sep="")
  }
  plotDataMatrix(qualityParametersList$slideDistance, labels=slideLabels, title=title, plotOutput=plotOutput, fileName=fileName)
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=16, fileName); dev.off()
  }


# normalisation diagnostics
if( ! missing(normObject) ){

  if( ! is.null(normObject) ){
    for( i in (1:length(normObject)) ){
      baseName <- paste("meanVsSDafterNormalisation",i,".",suffix,sep="")
      if( ! missing(savegraphicspath) ){
        fileName <- file.path(savegraphicspath, baseName)
      }else{
        fileName <- file.path(baseName)
      }
      if( class(normObject[[i]]) == "exprSet" ){
        if( plotOutput == "screen" ){
          x11()
        }else if( plotOutput == "pdf" ){
          pdf(file=fileName)
        }else{
          stop(" unexpected case; unknown plotOutput in qualityDiagnostics")
        }
        meanSdPlot(normObject[[i]], ranks=TRUE)
        if( plotOutput=="screen" & ! missing(savegraphicspath) ){
          dev.copy(device, fileName); dev.off()
        }
      }else{
        cat(" normObject is not of type exprSet in qualityDiagnostics \n")
        # possibly use plotMA for each hybridisation or
        #          use plotPrintTipLoess for each hybridisation
        #          (which requires the layout, available ?)
      }
    }
  }else{
    cat(" no normObject given in qualityDiagnostics \n")
  }

}
  
  # distributions 
  title <- "distributionOfRawDataChannelWise"
  plotDistributions(dataMatrix=
                    cbind(getIntensities(arrayDataObject)[,"Cy3",],
                          getIntensities(arrayDataObject)[,"Cy5",]),
                    colourVector=c(rep("darkgreen",length(slideLabels)),
                      rep("darkred",length(slideLabels))),
                    title=title,
                    plotOutput=plotOutput,
                    labels=c(slideLabels,slideLabels)) # i.e. two times slideLabels
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste(title,".",suffix,sep=""))); dev.off()
  }
  
  

  title <- "distributionOfTopRawDataChannelWise"
  plotDistributions(dataMatrix=
                    cbind(getIntensities(arrayDataObject)[,"Cy3",],
                          getIntensities(arrayDataObject)[,"Cy5",]),
                    #transFunc=log,
                    quantiles=c(0.75,0.99),
                    colourVector=c(rep("darkgreen",length(slideLabels)),
                      rep("darkred",length(slideLabels))),
                    title=title,
                    plotOutput=plotOutput,
                    labels=c(slideLabels,slideLabels)) # i.e. two times slideLabels
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste(title,".",suffix,sep=""))); dev.off()
  }
  
  
  title <- "distributionOfRawDataSlideWise"
  plotDistributions(dataMatrix=
                    interweave(getIntensities(arrayDataObject)[,"Cy3",],
                               getIntensities(arrayDataObject)[,"Cy5",]),
                    colourVector=rep(c("darkgreen","darkred"),length(slideLabels)),
                    title=title,
                    plotOutput=plotOutput,
                    labels=interweave(slideLabels,slideLabels))# i.e. two times slideLabels
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste(title,".",suffix,sep=""))); dev.off()
  }

  
  title <- "distributionOfTopDataSlideWise"
  plotDistributions(dataMatrix=
                    interweave(getIntensities(arrayDataObject)[,"Cy3",],
                               getIntensities(arrayDataObject)[,"Cy5",]),
                                        #transFunc=log,
                    quantiles=c(0.75,0.99),
                    colourVector=rep(c("darkgreen","darkred"),length(slideLabels)),
                    title=title,
                    plotOutput=plotOutput,
                    labels=interweave(slideLabels,slideLabels))# i.e. two times slideLabels
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste(title,".",suffix,sep=""))); dev.off()
  }
  
    title <- "distributionOfNormalisedData"
    plotDistributions(dataMatrix=
                      cbind(exprs(getExprSetCy3(exprSetArrayObject)),
                            exprs(getExprSetCy5(exprSetArrayObject))),
                      colourVector=c(rep("darkgreen",length(slideLabels)),
                        rep("darkred",length(slideLabels))),
                      title=title,
                      plotOutput=plotOutput,
                      labels=c(slideLabels,slideLabels)) # i.e. two times slideLabels
  if( plotOutput=="screen" & !missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste(title,".",suffix,sep=""))); dev.off()
  }


}# end of qualityDiagnostics
