# /**
#
# \name{visualiseQualityParameters}
#
# \title{visualiseQualityParameters}
#
# \alias{visualiseQualityParameters}
#
# \keyword{}
#
# \description{}
#
# \value{}
#
#
# \arguments{
#  \item{qualityParametersList}{ see \code{\link{qualityParameters}}}
#  \item{savegraphicspath}{}
#  \item{plotOutput}{character string specifying output plotOutput; either
#       "screen" or "pdf"; default: "screen"}
# }
# 
# \details{}
#
# \seealso{\code{}}
#
# \examples{
#	}
#
# \keyword{}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
# */

visualiseQualityParameters <- function(qualityParametersList, savegraphicspath, plotOutput="screen"){

  if( ! plotOutput %in% c("screen","pdf") ){
    warning(" unknown plotOutput in visualiseQualityParameters", call.=FALSE)
    return()
  }

  # so far in any case
  device <- pdf
  suffix <- "pdf"
  
  width <- 12
  height <- 12
  
  changeScale <- function(x,minimum,maximum){
    if( missing(minimum) ){
      minimum <- min(x)
    }
    if( missing(maximum) ){
      maximum <- max(x)
    }
    return((x-minimum) / (maximum-minimum))
  }

  if( plotOutput == "screen" ){
    x11(width=width,height=height)
  }else if( plotOutput == "pdf" ){
    pdf(width=width,height=height,file="visualiseQualityParametersOutput.pdf")
  }else{
    stop(" unexpected case; unknown plotOutput in visualiseQualityParameters ")
  }
  
  lo <- layout(matrix(data=c(1,1,1,2,3,4),nrow=6,ncol=1))
  
  h <- qualityParametersList$hybridisation
  i <- qualityParametersList$meanIntensity
  i3<- qualityParametersList$meanIntensityCy3
  i5<- qualityParametersList$meanIntensityCy5
  mi <- min(c(i,i3,i5))
  ma <- max(c(i,i3,i5))
  plot(1:length(h), main="intensity",ylim=c(0,1),changeScale(i,mi,ma), axes=FALSE, pch=19, col="black", xlab="", ylab="") #pch=20
  par(new=TRUE)
  lines(1:length(h),changeScale(i3,mi,ma), lwd=2, col="darkgreen")
  par(new=TRUE)
  lines(1:length(h),changeScale(i5,mi,ma), lwd=2, col="darkred")
  
  w <- qualityParametersList$width 
  d <- qualityParametersList$medianDistance
  c <- qualityParametersList$correlation

  plot(1:length(h), type="b", main="width",ylim=c(0,1),changeScale(w), axes=FALSE, pch=20, col="black", xlab="", ylab="good - bad") #pch=20
  plot(1:length(h), type="b", main="medianDistance",ylim=c(0,1),changeScale(d), axes=FALSE, pch=20, col="black", xlab="", ylab="good - bad") #pch=20
  plot(1:length(h), type="b", main="correlation",ylim=c(0,1),c, axes=FALSE, pch=20, col="black", xlab="", ylab="bad - good") #pch=20
  abline(1,0,col="blue")
  
  if( ! missing(savegraphicspath) ){
    dev.copy(device=device, width=20, height=20, file.path(savegraphicspath,paste("qualityResult",".",suffix,sep=""))); dev.off()
  }
  
  
}
